@@ -1,5 +1,5 @@
 /*
- * MUSCLE SmartCard Development ( http://pcsclite.alioth.debian.org/pcsclite.html )
+ * MUSCLE SmartCard Development ( https://pcsclite.apdu.fr/ )
  *
  * Copyright (C) 1999-2002
  *  David Corcoran <corcoran@musclecard.com>
@@ -28,8 +28,6 @@ DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
- *
- * $Id: debuglog.c 7004 2014-10-02 09:26:36Z rousseau $
  */
 
 /**
@@ -50,12 +48,12 @@ THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 #include <sys/types.h>
 #include <sys/time.h>
 #include <time.h>
+#include <pthread.h>
 
 #include "pcsclite.h"
 #include "misc.h"
 #include "debuglog.h"
 #include "sys_generic.h"
-#include "strlcpycat.h"
 
 #ifdef NO_LOG
 
@@ -84,11 +82,9 @@ void DebugLogSetLevel(const int level)
 	(void)level;
 }
 
-INTERNAL int DebugLogSetCategory(const int dbginfo)
+INTERNAL void DebugLogSetCategory(const int dbginfo)
 {
 	(void)dbginfo;
-
-	return 0;
 }
 
 INTERNAL void DebugLogCategory(const int category, const unsigned char *buffer,
@@ -114,7 +110,27 @@ static char LogLevel = PCSC_LOG_ERROR;
 
 static signed char LogDoColor = 0;	/**< no color by default */
 
-static void log_line(const int priority, const char *DebugBuffer);
+static void log_line(const int priority, const char *DebugBuffer,
+	unsigned int rv);
+
+/*
+ * log a message with the RV value returned by the daemon
+ */
+void log_msg_rv(const int priority, unsigned int rv, const char *fmt, ...)
+{
+	char DebugBuffer[DEBUG_BUF_SIZE];
+	va_list argptr;
+
+	if ((priority < LogLevel) /* log priority lower than threshold? */
+		|| (DEBUGLOG_NO_DEBUG == LogMsgType))
+		return;
+
+	va_start(argptr, fmt);
+	vsnprintf(DebugBuffer, sizeof DebugBuffer, fmt, argptr);
+	va_end(argptr);
+
+	log_line(priority, DebugBuffer, rv);
+}
 
 void log_msg(const int priority, const char *fmt, ...)
 {
@@ -129,10 +145,62 @@ void log_msg(const int priority, const char *fmt, ...)
 	vsnprintf(DebugBuffer, sizeof DebugBuffer, fmt, argptr);
 	va_end(argptr);
 
-	log_line(priority, DebugBuffer);
+	log_line(priority, DebugBuffer, -1);
 } /* log_msg */
 
-static void log_line(const int priority, const char *DebugBuffer)
+/* convert from integer rv value to a string value
+ * SCARD_S_SUCCESS -> "SCARD_S_SUCCESS"
+ */
+const char * rv2text(unsigned int rv)
+{
+	const char *rv_text = NULL;
+	static __thread char strError[30];
+
+#define CASE(x) \
+		case x: \
+			rv_text = "rv=" #x; \
+			break
+
+	if (rv != (unsigned int)-1)
+	{
+		switch (rv)
+		{
+			CASE(SCARD_S_SUCCESS);
+			CASE(SCARD_E_CANCELLED);
+			CASE(SCARD_E_INSUFFICIENT_BUFFER);
+			CASE(SCARD_E_INVALID_HANDLE);
+			CASE(SCARD_E_INVALID_PARAMETER);
+			CASE(SCARD_E_INVALID_VALUE);
+			CASE(SCARD_E_NO_MEMORY);
+			CASE(SCARD_E_NO_SERVICE);
+			CASE(SCARD_E_NO_SMARTCARD);
+			CASE(SCARD_E_NOT_TRANSACTED);
+			CASE(SCARD_E_PROTO_MISMATCH);
+			CASE(SCARD_E_READER_UNAVAILABLE);
+			CASE(SCARD_E_SHARING_VIOLATION);
+			CASE(SCARD_E_TIMEOUT);
+			CASE(SCARD_E_UNKNOWN_READER);
+			CASE(SCARD_E_UNSUPPORTED_FEATURE);
+			CASE(SCARD_F_COMM_ERROR);
+			CASE(SCARD_F_INTERNAL_ERROR);
+			CASE(SCARD_W_REMOVED_CARD);
+			CASE(SCARD_W_RESET_CARD);
+			CASE(SCARD_W_UNPOWERED_CARD);
+			CASE(SCARD_W_UNRESPONSIVE_CARD);
+			CASE(SCARD_E_NO_READERS_AVAILABLE);
+
+			default:
+				(void)snprintf(strError, sizeof(strError)-1,
+					"Unknown error: 0x%08X", rv);
+				rv_text = strError;
+		}
+	}
+
+	return rv_text;
+}
+
+static void log_line(const int priority, const char *DebugBuffer,
+	unsigned int rv)
 {
 	if (DEBUGLOG_SYSLOG_DEBUG == LogMsgType)
 		syslog(LOG_INFO, "%s", DebugBuffer);
@@ -142,6 +210,8 @@ static void log_line(const int priority, const char *DebugBuffer)
 		struct timeval new_time = { 0, 0 };
 		struct timeval tmp;
 		int delta;
+		pthread_t thread_id;
+		const char *rv_text = NULL;
 
 		gettimeofday(&new_time, NULL);
 		if (0 == last_time.tv_sec)
@@ -161,6 +231,10 @@ static void log_line(const int priority, const char *DebugBuffer)
 
 		last_time = new_time;
 
+		thread_id = pthread_self();
+
+		rv_text = rv2text(rv);
+
 		if (LogDoColor)
 		{
 			const char *color_pfx = "", *color_sfx = "\33[0m";
@@ -186,16 +260,40 @@ static void log_line(const int priority, const char *DebugBuffer)
 					break;
 			}
 
-			printf("%s%.8d%s %s%s%s\n", time_pfx, delta, time_sfx,
-				color_pfx, DebugBuffer, color_sfx);
+#ifdef __APPLE__
+#define THREAD_FORMAT "%p"
+#else
+#define THREAD_FORMAT "%lu"
+#endif
+			if (rv_text)
+			{
+				const char * rv_pfx = "", * rv_sfx = "";
+				if (rv != SCARD_S_SUCCESS)
+				{
+					rv_pfx = "\33[31m"; /* Red */
+					rv_sfx = "\33[0m";
+				}
+
+				printf("%s%.8d%s [" THREAD_FORMAT "] %s%s%s, %s%s%s\n",
+					time_pfx, delta, time_sfx, thread_id,
+					color_pfx, DebugBuffer, color_sfx,
+					rv_pfx, rv_text, rv_sfx);
+			}
+			else
+				printf("%s%.8d%s [" THREAD_FORMAT "] %s%s%s\n",
+					time_pfx, delta, time_sfx, thread_id,
+					color_pfx, DebugBuffer, color_sfx);
 		}
 		else
 		{
-			printf("%.8d %s\n", delta, DebugBuffer);
+			if (rv_text)
+				printf("%.8d %s, %s\n", delta, DebugBuffer, rv_text);
+			else
+				printf("%.8d %s\n", delta, DebugBuffer);
 		}
 		fflush(stdout);
 	}
-} /* log_msg */
+} /* log_line */
 
 static void log_xxd_always(const int priority, const char *msg,
 	const unsigned char *buffer, const int len)
@@ -203,10 +301,10 @@ static void log_xxd_always(const int priority, const char *msg,
 	char DebugBuffer[len*3 + strlen(msg) +1];
 	int i;
 	char *c;
-	size_t l;
 
-	l = strlcpy(DebugBuffer, msg, sizeof(DebugBuffer));
-	c = DebugBuffer + l;
+	/* DebugBuffer is always big enough for msg */
+	strcpy(DebugBuffer, msg);
+	c = DebugBuffer + strlen(DebugBuffer);
 
 	for (i = 0; (i < len); ++i)
 	{
@@ -215,7 +313,7 @@ static void log_xxd_always(const int priority, const char *msg,
 		c += 3;
 	}
 
-	log_line(priority, DebugBuffer);
+	log_line(priority, DebugBuffer, -1);
 } /* log_xxd_always */
 
 void log_xxd(const int priority, const char *msg, const unsigned char *buffer,
@@ -285,7 +383,7 @@ void DebugLogSetLevel(const int level)
 			break;
 
 		case PCSC_LOG_INFO:
-			Log1(PCSC_LOG_INFO, "debug level=notice");
+			Log1(PCSC_LOG_INFO, "debug level=info");
 			break;
 
 		case PCSC_LOG_DEBUG:
@@ -294,16 +392,13 @@ void DebugLogSetLevel(const int level)
 
 		default:
 			LogLevel = PCSC_LOG_INFO;
-			Log2(PCSC_LOG_CRITICAL, "unknown level (%d), using level=notice",
+			Log2(PCSC_LOG_CRITICAL, "unknown level (%d), using level=info",
 				level);
 	}
 }
 
-INTERNAL int DebugLogSetCategory(const int dbginfo)
+INTERNAL void DebugLogSetCategory(const int dbginfo)
 {
-#define DEBUG_INFO_LENGTH 80
-	char text[DEBUG_INFO_LENGTH];
-
 	/* use a negative number to UNset
 	 * typically use ~DEBUG_CATEGORY_APDU
 	 */
@@ -312,15 +407,8 @@ INTERNAL int DebugLogSetCategory(const int dbginfo)
 	else
 		LogCategory |= dbginfo;
 
-	/* set to empty string */
-	text[0] = '\0';
-
 	if (LogCategory & DEBUG_CATEGORY_APDU)
-		strlcat(text, " APDU", sizeof(text));
-
-	Log2(PCSC_LOG_INFO, "Debug options:%s", text);
-
-	return LogCategory;
+		Log1(PCSC_LOG_INFO, "Debug options: APDU");
 }
 
 INTERNAL void DebugLogCategory(const int category, const unsigned char *buffer,
